(function() {
    'use strict';

    angular
        .module('userParticipationGraphicsReport')
        .component('userParticipationGraphicsReport', {
            templateUrl: '/user-participation-graphics-report.template.html',
            controller: UserParticipationGraphicsReportCtrl,
            bindings: {
                date: '<',
                ntes: '<',
            },
        });

    UserParticipationGraphicsReportCtrl.$inject = ['$scope'];

    function UserParticipationGraphicsReportCtrl ($scope) {

        var ctrl = this;

        ctrl.minimized = false;

        ctrl.chart1Data = [];
        ctrl.chart1DatasetOverride = [
            {
                backgroundColor: '#89C9D5',
                barThickness: 10
            },
            {
                backgroundColor: '#1B7EC6',
                barThickness: 10
            }
        ];
        ctrl.chart1Labels = [];
        ctrl.chart1Series = ['Ativos', 'Não ativos'];
        ctrl.chart1Options = {
            legend: {
                align: 'center',
                display: true,
                position: 'bottom'
            },
            plugins: {
                dataLabels: {
                    display: false
                }
            },
            scales: {
                xAxes: [{
                    stacked: true,
                }],
                yAxes: [{
                    stacked: true,
                    ticks: {
                        fontColor: 'black',
                        fontStyle: 'bold'
                    },
                }]
            },
        };

        ctrl.chart2Data = [];
        ctrl.chart2Labels = ['Ativos', 'Não ativos'];
        ctrl.chart2Options = {
            legend: {
                align: 'center',
                display: true,
                position: 'bottom'
            },
            plugins: {
                datalabels: {
                    color: '#fff',
                    display: true,
                    font: {
                        weight: 'bold',
                    },
                    formatter: (value, ctx) => {
                        const dataArr = ctx.chart.data.datasets[0].data;
                        const sum = dataArr.reduce((acc, curr) => acc + curr, 0);
                        const percentage = Math.round(value * 100 / sum) + "%";
                        return percentage;
                    }
                }
            }
        };

        ctrl.toggleDetails = toggleDetails;

        ctrl.$onChanges = function() {
            ctrl.chart1Data = [
                ctrl.ntes.map(function(nte) {
                    return nte.activity_summary.actives;
                }),
                ctrl.ntes.map(function(nte) {
                    return nte.activity_summary.not_actives;
                })
            ];
            ctrl.chart1Labels = ctrl.ntes.map(function(nte) {
                return nte.name;
            });

            ctrl.chart2Data = ctrl.ntes.reduce(function (acc, nte) {
                return [
                    acc[0] + nte.activity_summary.actives,
                    acc[1] + nte.activity_summary.not_actives
                ];
            }, [0, 0]);
        };

        function toggleDetails() {
            ctrl.minimized = !ctrl.minimized;
        }
    }
})();
